@@ -1,6 +1,6 @@
 /* tracking.c - Client side caching: keys tracking and invalidation
  *
- * Copyright (c) 2019, Salvatore Sanfilippo <antirez at gmail dot com>
+ * Copyright (c) 2019, Redis Ltd.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,9 +53,9 @@ robj *TrackingChannelName;
  * represents the list of keys modified, and the list of clients that need
  * to be notified, for a given prefix. */
 typedef struct bcastState {
-    rax *keys;      /* Keys modified in the current event loop cycle. */
-    rax *clients;   /* Clients subscribed to the notification events for this
-                       prefix. */
+    rax *keys;    /* Keys modified in the current event loop cycle. */
+    rax *clients; /* Clients subscribed to the notification events for this
+                     prefix. */
 } bcastState;
 
 /* Remove the tracking state from the client 'c'. Note that there is not much
@@ -67,21 +67,23 @@ typedef struct bcastState {
 void disableTracking(client *c) {
     /* If this client is in broadcasting mode, we need to unsubscribe it
      * from all the prefixes it is registered to. */
-    if (c->flags & CLIENT_TRACKING_BCAST) {
+    if (c->flag.tracking_bcast) {
         raxIterator ri;
-        raxStart(&ri,c->client_tracking_prefixes);
-        raxSeek(&ri,"^",NULL,0);
-        while(raxNext(&ri)) {
-            bcastState *bs = raxFind(PrefixTable,ri.key,ri.key_len);
-            serverAssert(bs != raxNotFound);
-            raxRemove(bs->clients,(unsigned char*)&c,sizeof(c),NULL);
+        raxStart(&ri, c->client_tracking_prefixes);
+        raxSeek(&ri, "^", NULL, 0);
+        while (raxNext(&ri)) {
+            void *result;
+            int found = raxFind(PrefixTable, ri.key, ri.key_len, &result);
+            serverAssert(found);
+            bcastState *bs = result;
+            raxRemove(bs->clients, (unsigned char *)&c, sizeof(c), NULL);
             /* Was it the last client? Remove the prefix from the
              * table. */
             if (raxSize(bs->clients) == 0) {
                 raxFree(bs->clients);
                 raxFree(bs->keys);
                 zfree(bs);
-                raxRemove(PrefixTable,ri.key,ri.key_len,NULL);
+                raxRemove(PrefixTable, ri.key, ri.key_len, NULL);
             }
         }
         raxStop(&ri);
@@ -90,42 +92,42 @@ void disableTracking(client *c) {
     }
 
     /* Clear flags and adjust the count. */
-    if (c->flags & CLIENT_TRACKING) {
+    if (c->flag.tracking) {
         server.tracking_clients--;
-        c->flags &= ~(CLIENT_TRACKING|CLIENT_TRACKING_BROKEN_REDIR|
-                      CLIENT_TRACKING_BCAST|CLIENT_TRACKING_OPTIN|
-                      CLIENT_TRACKING_OPTOUT|CLIENT_TRACKING_CACHING|
-                      CLIENT_TRACKING_NOLOOP);
+        c->flag.tracking = 0;
+        c->flag.tracking_broken_redir = 0;
+        c->flag.tracking_bcast = 0;
+        c->flag.tracking_optin = 0;
+        c->flag.tracking_optout = 0;
+        c->flag.tracking_caching = 0;
+        c->flag.tracking_noloop = 0;
     }
 }
 
 static int stringCheckPrefix(unsigned char *s1, size_t s1_len, unsigned char *s2, size_t s2_len) {
     size_t min_length = s1_len < s2_len ? s1_len : s2_len;
-    return memcmp(s1,s2,min_length) == 0;   
+    return memcmp(s1, s2, min_length) == 0;
 }
 
 /* Check if any of the provided prefixes collide with one another or
- * with an existing prefix for the client. A collision is defined as two 
- * prefixes that will emit an invalidation for the same key. If no prefix 
- * collision is found, 1 is return, otherwise 0 is returned and the client 
+ * with an existing prefix for the client. A collision is defined as two
+ * prefixes that will emit an invalidation for the same key. If no prefix
+ * collision is found, 1 is return, otherwise 0 is returned and the client
  * has an error emitted describing the error. */
 int checkPrefixCollisionsOrReply(client *c, robj **prefixes, size_t numprefix) {
     for (size_t i = 0; i < numprefix; i++) {
         /* Check input list has no overlap with existing prefixes. */
         if (c->client_tracking_prefixes) {
             raxIterator ri;
-            raxStart(&ri,c->client_tracking_prefixes);
-            raxSeek(&ri,"^",NULL,0);
-            while(raxNext(&ri)) {
-                if (stringCheckPrefix(ri.key,ri.key_len,
-                    prefixes[i]->ptr,sdslen(prefixes[i]->ptr))) 
-                {
-                    sds collision = sdsnewlen(ri.key,ri.key_len);
+            raxStart(&ri, c->client_tracking_prefixes);
+            raxSeek(&ri, "^", NULL, 0);
+            while (raxNext(&ri)) {
+                if (stringCheckPrefix(ri.key, ri.key_len, prefixes[i]->ptr, sdslen(prefixes[i]->ptr))) {
+                    sds collision = sdsnewlen(ri.key, ri.key_len);
                     addReplyErrorFormat(c,
-                        "Prefix '%s' overlaps with an existing prefix '%s'. "
-                        "Prefixes for a single client must not overlap.",
-                        (unsigned char *)prefixes[i]->ptr,
-                        (unsigned char *)collision);
+                                        "Prefix '%s' overlaps with an existing prefix '%s'. "
+                                        "Prefixes for a single client must not overlap.",
+                                        (unsigned char *)prefixes[i]->ptr, (unsigned char *)collision);
                     sdsfree(collision);
                     raxStop(&ri);
                     return 0;
@@ -135,14 +137,12 @@ int checkPrefixCollisionsOrReply(client *c, robj **prefixes, size_t numprefix) {
         }
         /* Check input has no overlap with itself. */
         for (size_t j = i + 1; j < numprefix; j++) {
-            if (stringCheckPrefix(prefixes[i]->ptr,sdslen(prefixes[i]->ptr),
-                prefixes[j]->ptr,sdslen(prefixes[j]->ptr)))
-            {
+            if (stringCheckPrefix(prefixes[i]->ptr, sdslen(prefixes[i]->ptr), prefixes[j]->ptr,
+                                  sdslen(prefixes[j]->ptr))) {
                 addReplyErrorFormat(c,
-                    "Prefix '%s' overlaps with another provided prefix '%s'. "
-                    "Prefixes for a single client must not overlap.",
-                    (unsigned char *)prefixes[i]->ptr,
-                    (unsigned char *)prefixes[j]->ptr);
+                                    "Prefix '%s' overlaps with another provided prefix '%s'. "
+                                    "Prefixes for a single client must not overlap.",
+                                    (unsigned char *)prefixes[i]->ptr, (unsigned char *)prefixes[j]->ptr);
                 return i;
             }
         }
@@ -153,20 +153,21 @@ int checkPrefixCollisionsOrReply(client *c, robj **prefixes, size_t numprefix) {
 /* Set the client 'c' to track the prefix 'prefix'. If the client 'c' is
  * already registered for the specified prefix, no operation is performed. */
 void enableBcastTrackingForPrefix(client *c, char *prefix, size_t plen) {
-    bcastState *bs = raxFind(PrefixTable,(unsigned char*)prefix,plen);
+    void *result;
+    bcastState *bs;
     /* If this is the first client subscribing to such prefix, create
      * the prefix in the table. */
-    if (bs == raxNotFound) {
+    if (!raxFind(PrefixTable, (unsigned char *)prefix, plen, &result)) {
         bs = zmalloc(sizeof(*bs));
         bs->keys = raxNew();
         bs->clients = raxNew();
-        raxInsert(PrefixTable,(unsigned char*)prefix,plen,bs,NULL);
+        raxInsert(PrefixTable, (unsigned char *)prefix, plen, bs, NULL);
+    } else {
+        bs = result;
     }
-    if (raxTryInsert(bs->clients,(unsigned char*)&c,sizeof(c),NULL,NULL)) {
-        if (c->client_tracking_prefixes == NULL)
-            c->client_tracking_prefixes = raxNew();
-        raxInsert(c->client_tracking_prefixes,
-                  (unsigned char*)prefix,plen,NULL,NULL);
+    if (raxTryInsert(bs->clients, (unsigned char *)&c, sizeof(c), NULL, NULL)) {
+        if (c->client_tracking_prefixes == NULL) c->client_tracking_prefixes = raxNew();
+        raxInsert(c->client_tracking_prefixes, (unsigned char *)prefix, plen, NULL, NULL);
     }
 }
 
@@ -177,12 +178,14 @@ void enableBcastTrackingForPrefix(client *c, char *prefix, size_t plen) {
  * eventually get freed, we'll send a message to the original client to
  * inform it of the condition. Multiple clients can redirect the invalidation
  * messages to the same client ID. */
-void enableTracking(client *c, uint64_t redirect_to, uint64_t options, robj **prefix, size_t numprefix) {
-    if (!(c->flags & CLIENT_TRACKING)) server.tracking_clients++;
-    c->flags |= CLIENT_TRACKING;
-    c->flags &= ~(CLIENT_TRACKING_BROKEN_REDIR|CLIENT_TRACKING_BCAST|
-                  CLIENT_TRACKING_OPTIN|CLIENT_TRACKING_OPTOUT|
-                  CLIENT_TRACKING_NOLOOP);
+void enableTracking(client *c, uint64_t redirect_to, struct ClientFlags options, robj **prefix, size_t numprefix) {
+    if (!c->flag.tracking) server.tracking_clients++;
+    c->flag.tracking = 1;
+    c->flag.tracking_broken_redir = 0;
+    c->flag.tracking_bcast = 0;
+    c->flag.tracking_optin = 0;
+    c->flag.tracking_optout = 0;
+    c->flag.tracking_noloop = 0;
     c->client_tracking_redirection = redirect_to;
 
     /* This may be the first client we ever enable. Create the tracking
@@ -190,40 +193,42 @@ void enableTracking(client *c, uint64_t redirect_to, uint64_t options, robj **pr
     if (TrackingTable == NULL) {
         TrackingTable = raxNew();
         PrefixTable = raxNew();
-        TrackingChannelName = createStringObject("__redis__:invalidate",20);
+        TrackingChannelName = createStringObject("__redis__:invalidate", 20);
     }
 
     /* For broadcasting, set the list of prefixes in the client. */
-    if (options & CLIENT_TRACKING_BCAST) {
-        c->flags |= CLIENT_TRACKING_BCAST;
-        if (numprefix == 0) enableBcastTrackingForPrefix(c,"",0);
+    if (options.tracking_bcast) {
+        c->flag.tracking_bcast = 1;
+        if (numprefix == 0) enableBcastTrackingForPrefix(c, "", 0);
         for (size_t j = 0; j < numprefix; j++) {
             sds sdsprefix = prefix[j]->ptr;
-            enableBcastTrackingForPrefix(c,sdsprefix,sdslen(sdsprefix));
+            enableBcastTrackingForPrefix(c, sdsprefix, sdslen(sdsprefix));
         }
     }
 
     /* Set the remaining flags that don't need any special handling. */
-    c->flags |= options & (CLIENT_TRACKING_OPTIN|CLIENT_TRACKING_OPTOUT|
-                           CLIENT_TRACKING_NOLOOP);
+    c->flag.tracking_optin = options.tracking_optin;
+    c->flag.tracking_optout = options.tracking_optout;
+    c->flag.tracking_noloop = options.tracking_noloop;
 }
 
 /* This function is called after the execution of a readonly command in the
  * case the client 'c' has keys tracking enabled and the tracking is not
  * in BCAST mode. It will populate the tracking invalidation table according
- * to the keys the user fetched, so that Redis will know what are the clients
+ * to the keys the user fetched, so that the server will know what are the clients
  * that should receive an invalidation message with certain groups of keys
  * are modified. */
 void trackingRememberKeys(client *tracking, client *executing) {
     /* Return if we are in optin/out mode and the right CACHING command
      * was/wasn't given in order to modify the default behavior. */
-    uint64_t optin = tracking->flags & CLIENT_TRACKING_OPTIN;
-    uint64_t optout = tracking->flags & CLIENT_TRACKING_OPTOUT;
-    uint64_t caching_given = tracking->flags & CLIENT_TRACKING_CACHING;
+    uint64_t optin = tracking->flag.tracking_optin;
+    uint64_t optout = tracking->flag.tracking_optout;
+    uint64_t caching_given = tracking->flag.tracking_caching;
     if ((optin && !caching_given) || (optout && caching_given)) return;
 
-    getKeysResult result = GETKEYS_RESULT_INIT;
-    int numkeys = getKeysFromCommand(executing->cmd,executing->argv,executing->argc,&result);
+    getKeysResult result;
+    initGetKeysResult(&result);
+    int numkeys = getKeysFromCommand(executing->cmd, executing->argv, executing->argc, &result);
     if (!numkeys) {
         getKeysFreeResult(&result);
         return;
@@ -237,17 +242,19 @@ void trackingRememberKeys(client *tracking, client *executing) {
 
     keyReference *keys = result.keys;
 
-    for(int j = 0; j < numkeys; j++) {
+    for (int j = 0; j < numkeys; j++) {
         int idx = keys[j].pos;
         sds sdskey = executing->argv[idx]->ptr;
-        rax *ids = raxFind(TrackingTable,(unsigned char*)sdskey,sdslen(sdskey));
-        if (ids == raxNotFound) {
+        void *result;
+        rax *ids;
+        if (!raxFind(TrackingTable, (unsigned char *)sdskey, sdslen(sdskey), &result)) {
             ids = raxNew();
-            int inserted = raxTryInsert(TrackingTable,(unsigned char*)sdskey,
-                                        sdslen(sdskey),ids, NULL);
+            int inserted = raxTryInsert(TrackingTable, (unsigned char *)sdskey, sdslen(sdskey), ids, NULL);
             serverAssert(inserted == 1);
+        } else {
+            ids = result;
         }
-        if (raxTryInsert(ids,(unsigned char*)&tracking->id,sizeof(tracking->id),NULL,NULL))
+        if (raxTryInsert(ids, (unsigned char *)&tracking->id, sizeof(tracking->id), NULL, NULL))
             TrackingTableTotalItems++;
     }
     getKeysFreeResult(&result);
@@ -260,36 +267,36 @@ void trackingRememberKeys(client *tracking, client *executing) {
  *
  * In case the 'proto' argument is non zero, the function will assume that
  * 'keyname' points to a buffer of 'keylen' bytes already expressed in the
- * form of Redis RESP protocol. This is used for:
+ * form of RESP protocol. This is used for:
  * - In BCAST mode, to send an array of invalidated keys to all
  *   applicable clients
  * - Following a flush command, to send a single RESP NULL to indicate
  *   that all keys are now invalid. */
 void sendTrackingMessage(client *c, char *keyname, size_t keylen, int proto) {
-    uint64_t old_flags = c->flags;
-    c->flags |= CLIENT_PUSHING;
+    struct ClientFlags old_flags = c->flag;
+    c->flag.pushing = 1;
 
     int using_redirection = 0;
     if (c->client_tracking_redirection) {
         client *redir = lookupClientByID(c->client_tracking_redirection);
         if (!redir) {
-            c->flags |= CLIENT_TRACKING_BROKEN_REDIR;
+            c->flag.tracking_broken_redir = 1;
             /* We need to signal to the original connection that we
              * are unable to send invalidation messages to the redirected
              * connection, because the client no longer exist. */
             if (c->resp > 2) {
-                addReplyPushLen(c,2);
-                addReplyBulkCBuffer(c,"tracking-redir-broken",21);
-                addReplyLongLong(c,c->client_tracking_redirection);
+                addReplyPushLen(c, 2);
+                addReplyBulkCBuffer(c, "tracking-redir-broken", 21);
+                addReplyLongLong(c, c->client_tracking_redirection);
             }
-            if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+            if (!old_flags.pushing) c->flag.pushing = 0;
             return;
         }
-        if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        if (!old_flags.pushing) c->flag.pushing = 0;
         c = redir;
         using_redirection = 1;
-        old_flags = c->flags;
-        c->flags |= CLIENT_PUSHING;
+        old_flags = c->flag;
+        c->flag.pushing = 1;
     }
 
     /* Only send such info for clients in RESP version 3 or more. However
@@ -297,33 +304,33 @@ void sendTrackingMessage(client *c, char *keyname, size_t keylen, int proto) {
      * in Pub/Sub mode, we can support the feature with RESP 2 as well,
      * by sending Pub/Sub messages in the __redis__:invalidate channel. */
     if (c->resp > 2) {
-        addReplyPushLen(c,2);
-        addReplyBulkCBuffer(c,"invalidate",10);
-    } else if (using_redirection && c->flags & CLIENT_PUBSUB) {
+        addReplyPushLen(c, 2);
+        addReplyBulkCBuffer(c, "invalidate", 10);
+    } else if (using_redirection && c->flag.pubsub) {
         /* We use a static object to speedup things, however we assume
          * that addReplyPubsubMessage() will not take a reference. */
-        addReplyPubsubMessage(c,TrackingChannelName,NULL,shared.messagebulk);
+        addReplyPubsubMessage(c, TrackingChannelName, NULL, shared.messagebulk);
     } else {
         /* If are here, the client is not using RESP3, nor is
          * redirecting to another client. We can't send anything to
          * it since RESP2 does not support push messages in the same
          * connection. */
-        if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+        if (!old_flags.pushing) c->flag.pushing = 0;
         return;
     }
 
     /* Send the "value" part, which is the array of keys. */
     if (proto) {
-        addReplyProto(c,keyname,keylen);
+        addReplyProto(c, keyname, keylen);
     } else {
-        addReplyArrayLen(c,1);
-        addReplyBulkCBuffer(c,keyname,keylen);
+        addReplyArrayLen(c, 1);
+        addReplyBulkCBuffer(c, keyname, keylen);
     }
     updateClientMemUsageAndBucket(c);
-    if (!(old_flags & CLIENT_PUSHING)) c->flags &= ~CLIENT_PUSHING;
+    if (!old_flags.pushing) c->flag.pushing = 0;
 }
 
-/* This function is called when a key is modified in Redis and in the case
+/* This function is called when a key is modified in the server and in the case
  * we have at least one client with the BCAST mode enabled.
  * Its goal is to set the key in the right broadcast state if the key
  * matches one or more prefixes in the prefix table. Later when we
@@ -331,23 +338,22 @@ void sendTrackingMessage(client *c, char *keyname, size_t keylen, int proto) {
  * clients subscribed to each prefix. */
 void trackingRememberKeyToBroadcast(client *c, char *keyname, size_t keylen) {
     raxIterator ri;
-    raxStart(&ri,PrefixTable);
-    raxSeek(&ri,"^",NULL,0);
-    while(raxNext(&ri)) {
+    raxStart(&ri, PrefixTable);
+    raxSeek(&ri, "^", NULL, 0);
+    while (raxNext(&ri)) {
         if (ri.key_len > keylen) continue;
-        if (ri.key_len != 0 && memcmp(ri.key,keyname,ri.key_len) != 0)
-            continue;
+        if (ri.key_len != 0 && memcmp(ri.key, keyname, ri.key_len) != 0) continue;
         bcastState *bs = ri.data;
         /* We insert the client pointer as associated value in the radix
          * tree. This way we know who was the client that did the last
          * change to the key, and can avoid sending the notification in the
          * case the client is in NOLOOP mode. */
-        raxInsert(bs->keys,(unsigned char*)keyname,keylen,c,NULL);
+        raxInsert(bs->keys, (unsigned char *)keyname, keylen, c, NULL);
     }
     raxStop(&ri);
 }
 
-/* This function is called from signalModifiedKey() or other places in Redis
+/* This function is called from signalModifiedKey() or other places in the server
  * when a key changes value. In the context of keys tracking, our task here is
  * to send a notification to every client that may have keys about such caching
  * slot.
@@ -358,7 +364,7 @@ void trackingRememberKeyToBroadcast(client *c, char *keyname, size_t keylen) {
  *
  * The last argument 'bcast' tells the function if it should also schedule
  * the key for broadcasting to clients in BCAST mode. This is the case when
- * the function is called from the Redis core once a key is modified, however
+ * the function is called from the server core once a key is modified, however
  * we also call the function in order to evict keys in the key table in case
  * of memory pressure: in that case the key didn't really change, so we want
  * just to notify the clients that are in the table for this key, that would
@@ -366,50 +372,45 @@ void trackingRememberKeyToBroadcast(client *c, char *keyname, size_t keylen) {
 void trackingInvalidateKey(client *c, robj *keyobj, int bcast) {
     if (TrackingTable == NULL) return;
 
-    unsigned char *key = (unsigned char*)keyobj->ptr;
+    unsigned char *key = (unsigned char *)keyobj->ptr;
     size_t keylen = sdslen(keyobj->ptr);
 
-    if (bcast && raxSize(PrefixTable) > 0)
-        trackingRememberKeyToBroadcast(c,(char *)key,keylen);
+    if (bcast && raxSize(PrefixTable) > 0) trackingRememberKeyToBroadcast(c, (char *)key, keylen);
 
-    rax *ids = raxFind(TrackingTable,key,keylen);
-    if (ids == raxNotFound) return;
+    void *result;
+    if (!raxFind(TrackingTable, key, keylen, &result)) return;
+    rax *ids = result;
 
     raxIterator ri;
-    raxStart(&ri,ids);
-    raxSeek(&ri,"^",NULL,0);
-    while(raxNext(&ri)) {
+    raxStart(&ri, ids);
+    raxSeek(&ri, "^", NULL, 0);
+    while (raxNext(&ri)) {
         uint64_t id;
-        memcpy(&id,ri.key,sizeof(id));
+        memcpy(&id, ri.key, sizeof(id));
         client *target = lookupClientByID(id);
         /* Note that if the client is in BCAST mode, we don't want to
          * send invalidation messages that were pending in the case
          * previously the client was not in BCAST mode. This can happen if
          * TRACKING is enabled normally, and then the client switches to
          * BCAST mode. */
-        if (target == NULL ||
-            !(target->flags & CLIENT_TRACKING)||
-            target->flags & CLIENT_TRACKING_BCAST)
-        {
+        if (target == NULL || !(target->flag.tracking) || target->flag.tracking_bcast) {
             continue;
         }
 
         /* If the client enabled the NOLOOP mode, don't send notifications
          * about keys changed by the client itself. */
-        if (target->flags & CLIENT_TRACKING_NOLOOP &&
-            target == server.current_client)
-        {
+        if (target->flag.tracking_noloop && target == server.current_client) {
             continue;
         }
 
         /* If target is current client and it's executing a command, we need schedule key invalidation.
          * As the invalidation messages may be interleaved with command
          * response and should after command response. */
-        if (target == server.current_client && (server.current_client->flags & CLIENT_EXECUTING_COMMAND)) {
+        if (target == server.current_client && (server.current_client->flag.executing_command)) {
             incrRefCount(keyobj);
             listAddNodeTail(server.tracking_pending_keys, keyobj);
         } else {
-            sendTrackingMessage(target,(char *)keyobj->ptr,sdslen(keyobj->ptr),0);
+            sendTrackingMessage(target, (char *)keyobj->ptr, sdslen(keyobj->ptr), 0);
         }
     }
     raxStop(&ri);
@@ -418,7 +419,7 @@ void trackingInvalidateKey(client *c, robj *keyobj, int bcast) {
      * again if more keys will be modified in this caching slot. */
     TrackingTableTotalItems -= raxSize(ids);
     raxFree(ids);
-    raxRemove(TrackingTable,(unsigned char*)key,keylen,NULL);
+    raxRemove(TrackingTable, (unsigned char *)key, keylen, NULL);
 }
 
 void trackingHandlePendingKeyInvalidations(void) {
@@ -431,17 +432,17 @@ void trackingHandlePendingKeyInvalidations(void) {
     listNode *ln;
     listIter li;
 
-    listRewind(server.tracking_pending_keys,&li);
+    listRewind(server.tracking_pending_keys, &li);
     while ((ln = listNext(&li)) != NULL) {
         robj *key = listNodeValue(ln);
         /* current_client maybe freed, so we need to send invalidation
          * message only when current_client is still alive */
         if (server.current_client != NULL) {
             if (key != NULL) {
-                sendTrackingMessage(server.current_client,(char *)key->ptr,sdslen(key->ptr),0);
+                sendTrackingMessage(server.current_client, (char *)key->ptr, sdslen(key->ptr), 0);
             } else {
-                sendTrackingMessage(server.current_client,shared.null[server.current_client->resp]->ptr,
-                    sdslen(shared.null[server.current_client->resp]->ptr),1);
+                sendTrackingMessage(server.current_client, shared.null[server.current_client->resp]->ptr,
+                                    sdslen(shared.null[server.current_client->resp]->ptr), 1);
             }
         }
         if (key != NULL) decrRefCount(key);
@@ -449,11 +450,11 @@ void trackingHandlePendingKeyInvalidations(void) {
     listEmpty(server.tracking_pending_keys);
 }
 
-/* This function is called when one or all the Redis databases are
- * flushed. Caching keys are not specific for each DB but are global: 
- * currently what we do is send a special notification to clients with 
- * tracking enabled, sending a RESP NULL, which means, "all the keys", 
- * in order to avoid flooding clients with many invalidation messages 
+/* This function is called when one or all of the databases are
+ * flushed. Caching keys are not specific for each DB but are global:
+ * currently what we do is send a special notification to clients with
+ * tracking enabled, sending a RESP NULL, which means, "all the keys",
+ * in order to avoid flooding clients with many invalidation messages
  * for all the keys they may hold.
  */
 void freeTrackingRadixTreeCallback(void *rt) {
@@ -461,7 +462,7 @@ void freeTrackingRadixTreeCallback(void *rt) {
 }
 
 void freeTrackingRadixTree(rax *rt) {
-    raxFreeWithCallback(rt,freeTrackingRadixTreeCallback);
+    raxFreeWithCallback(rt, freeTrackingRadixTreeCallback);
 }
 
 /* A RESP NULL is sent to indicate that all keys are invalid */
@@ -469,15 +470,15 @@ void trackingInvalidateKeysOnFlush(int async) {
     if (server.tracking_clients) {
         listNode *ln;
         listIter li;
-        listRewind(server.clients,&li);
+        listRewind(server.clients, &li);
         while ((ln = listNext(&li)) != NULL) {
             client *c = listNodeValue(ln);
-            if (c->flags & CLIENT_TRACKING) {
+            if (c->flag.tracking) {
                 if (c == server.current_client) {
                     /* We use a special NULL to indicate that we should send null */
-                    listAddNodeTail(server.tracking_pending_keys,NULL);
+                    listAddNodeTail(server.tracking_pending_keys, NULL);
                 } else {
-                    sendTrackingMessage(c,shared.null[c->resp]->ptr,sdslen(shared.null[c->resp]->ptr),1);
+                    sendTrackingMessage(c, shared.null[c->resp]->ptr, sdslen(shared.null[c->resp]->ptr), 1);
                 }
             }
         }
@@ -495,12 +496,12 @@ void trackingInvalidateKeysOnFlush(int async) {
     }
 }
 
-/* Tracking forces Redis to remember information about which client may have
+/* Tracking forces the server to remember information about which client may have
  * certain keys. In workloads where there are a lot of reads, but keys are
  * hardly modified, the amount of information we have to remember server side
  * could be a lot, with the number of keys being totally not bound.
  *
- * So Redis allows the user to configure a maximum number of keys for the
+ * So the server allows the user to configure a maximum number of keys for the
  * invalidation table. This function makes sure that we don't go over the
  * specified fill rate: if we are over, we can just evict information about
  * a random key, and send invalidation messages to clients like if the key was
@@ -518,18 +519,18 @@ void trackingLimitUsedSlots(void) {
     /* We have to invalidate a few keys to reach the limit again. The effort
      * we do here is proportional to the number of times we entered this
      * function and found that we are still over the limit. */
-    int effort = 100 * (timeout_counter+1);
+    int effort = 100 * (timeout_counter + 1);
 
     /* We just remove one key after another by using a random walk. */
     raxIterator ri;
-    raxStart(&ri,TrackingTable);
-    while(effort > 0) {
+    raxStart(&ri, TrackingTable);
+    while (effort > 0) {
         effort--;
-        raxSeek(&ri,"^",NULL,0);
-        raxRandomWalk(&ri,0);
+        raxSeek(&ri, "^", NULL, 0);
+        raxRandomWalk(&ri, 0);
         if (raxEOF(&ri)) break;
-        robj *keyobj = createStringObject((char*)ri.key,ri.key_len);
-        trackingInvalidateKey(NULL,keyobj,0);
+        robj *keyobj = createStringObject((char *)ri.key, ri.key_len);
+        trackingInvalidateKey(NULL, keyobj, 0);
         decrRefCount(keyobj);
         if (raxSize(TrackingTable) <= max_keys) {
             timeout_counter = 0;
@@ -544,7 +545,7 @@ void trackingLimitUsedSlots(void) {
     timeout_counter++;
 }
 
-/* Generate Redis protocol for an array containing all the key names
+/* Generate RESP for an array containing all the key names
  * in the 'keys' radix tree. If the client is not NULL, the list will not
  * include keys that were modified the last time by this client, in order
  * to implement the NOLOOP option.
@@ -558,9 +559,9 @@ sds trackingBuildBroadcastReply(client *c, rax *keys) {
         count = raxSize(keys);
     } else {
         count = 0;
-        raxStart(&ri,keys);
-        raxSeek(&ri,"^",NULL,0);
-        while(raxNext(&ri)) {
+        raxStart(&ri, keys);
+        raxSeek(&ri, "^", NULL, 0);
+        while (raxNext(&ri)) {
             if (ri.data != c) count++;
         }
         raxStop(&ri);
@@ -569,24 +570,24 @@ sds trackingBuildBroadcastReply(client *c, rax *keys) {
     }
 
     /* Create the array reply with the list of keys once, then send
-    * it to all the clients subscribed to this prefix. */
+     * it to all the clients subscribed to this prefix. */
     char buf[32];
-    size_t len = ll2string(buf,sizeof(buf),count);
+    size_t len = ll2string(buf, sizeof(buf), count);
     sds proto = sdsempty();
-    proto = sdsMakeRoomFor(proto,count*15);
-    proto = sdscatlen(proto,"*",1);
-    proto = sdscatlen(proto,buf,len);
-    proto = sdscatlen(proto,"\r\n",2);
-    raxStart(&ri,keys);
-    raxSeek(&ri,"^",NULL,0);
-    while(raxNext(&ri)) {
+    proto = sdsMakeRoomFor(proto, count * 15);
+    proto = sdscatlen(proto, "*", 1);
+    proto = sdscatlen(proto, buf, len);
+    proto = sdscatlen(proto, "\r\n", 2);
+    raxStart(&ri, keys);
+    raxSeek(&ri, "^", NULL, 0);
+    while (raxNext(&ri)) {
         if (c && ri.data == c) continue;
-        len = ll2string(buf,sizeof(buf),ri.key_len);
-        proto = sdscatlen(proto,"$",1);
-        proto = sdscatlen(proto,buf,len);
-        proto = sdscatlen(proto,"\r\n",2);
-        proto = sdscatlen(proto,ri.key,ri.key_len);
-        proto = sdscatlen(proto,"\r\n",2);
+        len = ll2string(buf, sizeof(buf), ri.key_len);
+        proto = sdscatlen(proto, "$", 1);
+        proto = sdscatlen(proto, buf, len);
+        proto = sdscatlen(proto, "\r\n", 2);
+        proto = sdscatlen(proto, ri.key, ri.key_len);
+        proto = sdscatlen(proto, "\r\n", 2);
     }
     raxStop(&ri);
     return proto;
@@ -601,33 +602,33 @@ void trackingBroadcastInvalidationMessages(void) {
     /* Return ASAP if there is nothing to do here. */
     if (TrackingTable == NULL || !server.tracking_clients) return;
 
-    raxStart(&ri,PrefixTable);
-    raxSeek(&ri,"^",NULL,0);
+    raxStart(&ri, PrefixTable);
+    raxSeek(&ri, "^", NULL, 0);
 
     /* For each prefix... */
-    while(raxNext(&ri)) {
+    while (raxNext(&ri)) {
         bcastState *bs = ri.data;
 
         if (raxSize(bs->keys)) {
             /* Generate the common protocol for all the clients that are
              * not using the NOLOOP option. */
-            sds proto = trackingBuildBroadcastReply(NULL,bs->keys);
+            sds proto = trackingBuildBroadcastReply(NULL, bs->keys);
 
             /* Send this array of keys to every client in the list. */
-            raxStart(&ri2,bs->clients);
-            raxSeek(&ri2,"^",NULL,0);
-            while(raxNext(&ri2)) {
+            raxStart(&ri2, bs->clients);
+            raxSeek(&ri2, "^", NULL, 0);
+            while (raxNext(&ri2)) {
                 client *c;
-                memcpy(&c,ri2.key,sizeof(c));
-                if (c->flags & CLIENT_TRACKING_NOLOOP) {
+                memcpy(&c, ri2.key, sizeof(c));
+                if (c->flag.tracking_noloop) {
                     /* This client may have certain keys excluded. */
-                    sds adhoc = trackingBuildBroadcastReply(c,bs->keys);
+                    sds adhoc = trackingBuildBroadcastReply(c, bs->keys);
                     if (adhoc) {
-                        sendTrackingMessage(c,adhoc,sdslen(adhoc),1);
+                        sendTrackingMessage(c, adhoc, sdslen(adhoc), 1);
                         sdsfree(adhoc);
                     }
                 } else {
-                    sendTrackingMessage(c,proto,sdslen(proto),1);
+                    sendTrackingMessage(c, proto, sdslen(proto), 1);
                 }
             }
             raxStop(&ri2);
