@@ -1,95 +1,54 @@
-# check functionality compression of plain and zipped nodes
+# check functionality compression of plain and packed nodes
 start_server [list overrides [list save ""] ] {
     r config set list-compress-depth 2
     r config set list-max-ziplist-size 1
 
-    # 3 test to check compression with regular ziplist nodes
+    # 3 test to check compression with plain and packed nodes
     # 1. using push + insert
     # 2. using push + insert + trim
     # 3. using push + insert + set
 
-    test {reg node check compression with insert and pop} {
-        r lpush list1 [string repeat a 500]
-        r lpush list1 [string repeat b 500]
-        r lpush list1 [string repeat c 500]
-        r lpush list1 [string repeat d 500]
-        r linsert list1 after [string repeat d 500] [string repeat e 500]
-        r linsert list1 after [string repeat d 500] [string repeat f 500]
-        r linsert list1 after [string repeat d 500] [string repeat g 500]
-        r linsert list1 after [string repeat d 500] [string repeat j 500]
-        assert_equal [r lpop list1] [string repeat d 500]
-        assert_equal [r lpop list1] [string repeat j 500]
-        assert_equal [r lpop list1] [string repeat g 500]
-        assert_equal [r lpop list1] [string repeat f 500]
-        assert_equal [r lpop list1] [string repeat e 500]
-        assert_equal [r lpop list1] [string repeat c 500]
-        assert_equal [r lpop list1] [string repeat b 500]
-        assert_equal [r lpop list1] [string repeat a 500]
+    foreach {container size} {packed 500 plain 8193} {
+    test "$container node check compression with insert and pop" {
+        r flushdb
+        r lpush list1 [string repeat a $size]
+        r lpush list1 [string repeat b $size]
+        r lpush list1 [string repeat c $size]
+        r lpush list1 [string repeat d $size]
+        r linsert list1 after [string repeat d $size] [string repeat e $size]
+        r linsert list1 after [string repeat d $size] [string repeat f $size]
+        r linsert list1 after [string repeat d $size] [string repeat g $size]
+        r linsert list1 after [string repeat d $size] [string repeat j $size]
+        assert_equal [r lpop list1] [string repeat d $size]
+        assert_equal [r lpop list1] [string repeat j $size]
+        assert_equal [r lpop list1] [string repeat g $size]
+        assert_equal [r lpop list1] [string repeat f $size]
+        assert_equal [r lpop list1] [string repeat e $size]
+        assert_equal [r lpop list1] [string repeat c $size]
+        assert_equal [r lpop list1] [string repeat b $size]
+        assert_equal [r lpop list1] [string repeat a $size]
     };
 
-    test {reg node check compression combined with trim} {
-        r lpush list2 [string repeat a 500]
-        r linsert list2 after  [string repeat a 500] [string repeat b 500]
-        r rpush list2 [string repeat c 500]
-        assert_equal [string repeat b 500] [r lindex list2 1]
+    test "$container node check compression combined with trim" {
+        r flushdb
+        r lpush list2 [string repeat a $size]
+        r linsert list2 after  [string repeat a $size] [string repeat b $size]
+        r rpush list2 [string repeat c $size]
+        assert_equal [string repeat b $size] [r lindex list2 1]
         r LTRIM list2 1 -1
         r llen list2
     } {2}
 
-    test {reg node check compression with lset} {
-        r lpush list3 [string repeat a 500]
-        r LSET list3 0 [string repeat b 500]
-        assert_equal [string repeat b 500] [r lindex list3 0]
-        r lpush list3 [string repeat c 500]
-        r LSET list3 0 [string repeat d 500]
-        assert_equal [string repeat d 500] [r lindex list3 0]
+    test "$container node check compression with lset" {
+        r flushdb
+        r lpush list3 [string repeat a $size]
+        r LSET list3 0 [string repeat b $size]
+        assert_equal [string repeat b $size] [r lindex list3 0]
+        r lpush list3 [string repeat c $size]
+        r LSET list3 0 [string repeat d $size]
+        assert_equal [string repeat d $size] [r lindex list3 0]
     }
-
-    # repeating the 3 tests with plain nodes
-    # (by adjusting quicklist-packed-threshold)
-
-    test {plain node check compression} {
-        r debug quicklist-packed-threshold 1b
-        r lpush list4 [string repeat a 500]
-        r lpush list4 [string repeat b 500]
-        r lpush list4 [string repeat c 500]
-        r lpush list4 [string repeat d 500]
-        r linsert list4 after [string repeat d 500] [string repeat e 500]
-        r linsert list4 after [string repeat d 500] [string repeat f 500]
-        r linsert list4 after [string repeat d 500] [string repeat g 500]
-        r linsert list4 after [string repeat d 500] [string repeat j 500]
-        assert_equal [r lpop list4] [string repeat d 500]
-        assert_equal [r lpop list4] [string repeat j 500]
-        assert_equal [r lpop list4] [string repeat g 500]
-        assert_equal [r lpop list4] [string repeat f 500]
-        assert_equal [r lpop list4] [string repeat e 500]
-        assert_equal [r lpop list4] [string repeat c 500]
-        assert_equal [r lpop list4] [string repeat b 500]
-        assert_equal [r lpop list4] [string repeat a 500]
-        r debug quicklist-packed-threshold 0
-    } {OK} {needs:debug}
-
-    test {plain node check compression with ltrim} {
-        r debug quicklist-packed-threshold 1b
-        r lpush list5 [string repeat a 500]
-        r linsert list5 after  [string repeat a 500] [string repeat b 500]
-        r rpush list5 [string repeat c 500]
-        assert_equal [string repeat b 500] [r lindex list5 1]
-        r LTRIM list5 1 -1
-        assert_equal [r llen list5] 2
-        r debug quicklist-packed-threshold 0
-    } {OK} {needs:debug}
-
-    test {plain node check compression using lset} {
-        r debug quicklist-packed-threshold 1b
-        r lpush list6 [string repeat a 500]
-        r LSET list6 0 [string repeat b 500]
-        assert_equal [string repeat b 500] [r lindex list6 0]
-        r lpush list6 [string repeat c 500]
-        r LSET list6 0 [string repeat d 500]
-        assert_equal [string repeat d 500] [r lindex list6 0]
-        r debug quicklist-packed-threshold 0
-    } {OK} {needs:debug}
+    } ;# foreach
 
     # revert config for external mode tests.
     r config set list-compress-depth 0
@@ -97,6 +56,13 @@ start_server [list overrides [list save ""] ] {
 
 # check functionality of plain nodes using low packed-threshold
 start_server [list overrides [list save ""] ] {
+foreach type {listpack quicklist} {
+    if {$type eq "listpack"} {
+        r config set list-max-listpack-size -2
+    } else {
+        r config set list-max-listpack-size 1
+    }
+
     # basic command check for plain nodes - "LPUSH & LPOP"
     test {Test LPUSH and LPOP on plain nodes} {
         r flushdb
@@ -104,6 +70,7 @@ start_server [list overrides [list save ""] ] {
         r lpush lst 9
         r lpush lst xxxxxxxxxx
         r lpush lst xxxxxxxxxx
+        assert_encoding $type lst
         set s0 [s used_memory]
         assert {$s0 > 10}
         assert {[r llen lst] == 3}
@@ -128,6 +95,7 @@ start_server [list overrides [list save ""] ] {
         r lpush lst xxxxxxxxxxx
         r lpush lst 9
         r lpush lst xxxxxxxxxxx
+        assert_encoding $type lst
         r linsert lst before "9" "8"
         assert {[r lindex lst 1] eq "8"}
         r linsert lst BEFORE "9" "7"
@@ -143,6 +111,7 @@ start_server [list overrides [list save ""] ] {
         r lpush lst1 9
         r lpush lst1 xxxxxxxxxxx
         r lpush lst1 9
+        assert_encoding $type lst1
         r LTRIM lst1 1 -1
         assert_equal [r llen lst1] 2
         r debug quicklist-packed-threshold 0
@@ -154,6 +123,7 @@ start_server [list overrides [list save ""] ] {
         r debug quicklist-packed-threshold 1b
         r lpush lst one
         r lpush lst xxxxxxxxxxx
+        assert_encoding $type lst
         set s0 [s used_memory]
         assert {$s0 > 10}
         r lpush lst 9
@@ -169,6 +139,7 @@ start_server [list overrides [list save ""] ] {
         r RPUSH lst "aa"
         r RPUSH lst "bb"
         r RPUSH lst "cc"
+        assert_encoding $type lst
         r LSET lst 0 "xxxxxxxxxxx"
         assert_equal [r LPOS lst "xxxxxxxxxxx"] 0
         r debug quicklist-packed-threshold 0
@@ -180,6 +151,7 @@ start_server [list overrides [list save ""] ] {
         r debug quicklist-packed-threshold 1b
         r RPUSH lst2{t} "aa"
         r RPUSH lst2{t} "bb"
+        assert_encoding $type lst2{t}
         r LSET lst2{t} 0 xxxxxxxxxxx
         r RPUSH lst2{t} "cc"
         r RPUSH lst2{t} "dd"
@@ -200,6 +172,7 @@ start_server [list overrides [list save ""] ] {
         r debug quicklist-packed-threshold 5b
         r RPUSH lst "aa"
         r RPUSH lst "bb"
+        assert_encoding $type lst
         r lset lst 0 [string repeat d 50001]
         set s1 [r lpop lst]
         assert_equal $s1 [string repeat d 50001]
@@ -281,6 +254,7 @@ start_server [list overrides [list save ""] ] {
         # Insert two elements and keep them in the same node
         r RPUSH lst $small_ele
         r RPUSH lst $small_ele
+        assert_encoding $type lst
 
         # When setting the position of -1 to a large element, we first insert
         # a large element at the end and then delete its previous element.
@@ -297,6 +271,7 @@ start_server [list overrides [list save ""] ] {
 
         r LPUSH lst "aa"
         r LPUSH lst "bb"
+        assert_encoding $type lst
         r LSET lst -2 [string repeat x 10]
         r RPOP lst
         assert_equal [string repeat x 10] [r LRANGE lst 0 -1]
@@ -304,6 +279,7 @@ start_server [list overrides [list save ""] ] {
         r debug quicklist-packed-threshold 0
     } {OK} {needs:debug}
 }
+}
 
 run_solo {list-large-memory} {
 start_server [list overrides [list save ""] ] {
@@ -472,7 +448,7 @@ start_server {
         "list-max-ziplist-size" -1
     }
 } {
-    source "tests/unit/type/list-common.tcl"
+    array set largevalue [generate_largevalue_test_array]
 
     # A helper function to execute either B*POP or BLMPOP* with one input key.
     proc bpop_command {rd pop key timeout} {
@@ -669,7 +645,7 @@ foreach {type large} [array get largevalue] {
     foreach {type large} [array get largevalue] {
     foreach {pop} {BLPOP BLMPOP_LEFT} {
         test "$pop: single existing list - $type" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             create_$type blist "a b $large c d"
 
             bpop_command $rd $pop blist 1
@@ -695,7 +671,7 @@ foreach {type large} [array get largevalue] {
         }
 
         test "$pop: multiple existing lists - $type" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             create_$type blist1{t} "a $large c"
             create_$type blist2{t} "d $large f"
 
@@ -724,7 +700,7 @@ foreach {type large} [array get largevalue] {
         }
 
         test "$pop: second list has an entry - $type" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1{t}
             create_$type blist2{t} "d $large f"
 
@@ -746,7 +722,7 @@ foreach {type large} [array get largevalue] {
             r del target{t}
             r rpush target{t} bar
 
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             create_$type blist{t} "a b $large c d"
 
             $rd brpoplpush blist{t} target{t} 1
@@ -763,7 +739,7 @@ foreach {type large} [array get largevalue] {
                     r del target{t}
                     r rpush target{t} bar
 
-                    set rd [redis_deferring_client]
+                    set rd [valkey_deferring_client]
                     create_$type blist{t} "a b $large c d"
 
                     $rd blmove blist{t} target{t} $wherefrom $whereto 1
@@ -790,7 +766,7 @@ foreach {type large} [array get largevalue] {
 
 foreach {pop} {BLPOP BLMPOP_LEFT} {
     test "$pop, LPUSH + DEL should not awake blocked client" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del list
 
         bpop_command $rd $pop list 0
@@ -807,7 +783,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "$pop, LPUSH + DEL + SET should not awake blocked client" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del list
 
         bpop_command $rd $pop list 0
@@ -826,7 +802,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
 }
 
     test "BLPOP with same key multiple times should work (issue #801)" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del list1{t} list2{t}
 
         # Data arriving after the BLPOP.
@@ -851,7 +827,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
 
 foreach {pop} {BLPOP BLMPOP_LEFT} {
     test "MULTI/EXEC is isolated from the point of view of $pop" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del list
 
         bpop_command $rd $pop list 0
@@ -867,7 +843,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "$pop with variadic LPUSH" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist
         bpop_command $rd $pop blist 0
         wait_for_blocked_client
@@ -879,7 +855,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
 }
 
     test "BRPOPLPUSH with zero timeout should block indefinitely" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist{t} target{t}
         r rpush target{t} bar
         $rd brpoplpush blist{t} target{t} 0
@@ -893,7 +869,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     foreach wherefrom {left right} {
         foreach whereto {left right} {
             test "BLMOVE $wherefrom $whereto with zero timeout should block indefinitely" {
-                set rd [redis_deferring_client]
+                set rd [valkey_deferring_client]
                 r del blist{t} target{t}
                 r rpush target{t} bar
                 $rd blmove blist{t} target{t} $wherefrom $whereto 0
@@ -913,8 +889,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     foreach wherefrom {left right} {
         foreach whereto {left right} {
             test "BLMOVE ($wherefrom, $whereto) with a client BLPOPing the target list" {
-                set rd [redis_deferring_client]
-                set rd2 [redis_deferring_client]
+                set rd [valkey_deferring_client]
+                set rd2 [valkey_deferring_client]
                 r del blist{t} target{t}
                 $rd2 blpop target{t} 0
                 wait_for_blocked_clients_count 1
@@ -931,7 +907,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "BRPOPLPUSH with wrong source type" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist{t} target{t}
         r set blist{t} nolist
         $rd brpoplpush blist{t} target{t} 1
@@ -940,7 +916,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "BRPOPLPUSH with wrong destination type" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist{t} target{t}
         r set target{t} nolist
         r lpush blist{t} foo
@@ -948,7 +924,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         assert_error "WRONGTYPE*" {$rd read}
         $rd close
 
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist{t} target{t}
         r set target{t} nolist
         $rd brpoplpush blist{t} target{t} 0
@@ -960,7 +936,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "BRPOPLPUSH maintains order of elements after failure" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del blist{t} target{t}
         r set target{t} nolist
         $rd brpoplpush blist{t} target{t} 0
@@ -972,8 +948,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     } {a b c}
 
     test "BRPOPLPUSH with multiple blocked clients" {
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
         r del blist{t} target1{t} target2{t}
         r set target1{t} nolist
         $rd1 brpoplpush blist{t} target1{t} 0
@@ -990,10 +966,10 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "BLMPOP with multiple blocked clients" {
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
-        set rd3 [redis_deferring_client]
-        set rd4 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
+        set rd3 [valkey_deferring_client]
+        set rd4 [valkey_deferring_client]
         r del blist{t} blist2{t}
 
         $rd1 blmpop 0 2 blist{t} blist2{t} left count 1
@@ -1023,8 +999,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "Linked LMOVEs" {
-      set rd1 [redis_deferring_client]
-      set rd2 [redis_deferring_client]
+      set rd1 [valkey_deferring_client]
+      set rd2 [valkey_deferring_client]
 
       r del list1{t} list2{t} list3{t}
 
@@ -1043,8 +1019,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "Circular BRPOPLPUSH" {
-      set rd1 [redis_deferring_client]
-      set rd2 [redis_deferring_client]
+      set rd1 [valkey_deferring_client]
+      set rd2 [valkey_deferring_client]
 
       r del list1{t} list2{t}
 
@@ -1062,7 +1038,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     }
 
     test "Self-referential BRPOPLPUSH" {
-      set rd [redis_deferring_client]
+      set rd [valkey_deferring_client]
 
       r del blist{t}
 
@@ -1090,8 +1066,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     } {foo bar {} {} {bar foo}}
 
     test "PUSH resulting from BRPOPLPUSH affect WATCH" {
-        set blocked_client [redis_deferring_client]
-        set watching_client [redis_deferring_client]
+        set blocked_client [valkey_deferring_client]
+        set watching_client [valkey_deferring_client]
         r del srclist{t} dstlist{t} somekey{t}
         r set somekey{t} somevalue
         $blocked_client brpoplpush srclist{t} dstlist{t} 0
@@ -1111,8 +1087,8 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     } {}
 
     test "BRPOPLPUSH does not affect WATCH while still blocked" {
-        set blocked_client [redis_deferring_client]
-        set watching_client [redis_deferring_client]
+        set blocked_client [valkey_deferring_client]
+        set watching_client [valkey_deferring_client]
         r del srclist{t} dstlist{t} somekey{t}
         r set somekey{t} somevalue
         $blocked_client brpoplpush srclist{t} dstlist{t} 0
@@ -1124,6 +1100,13 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         $watching_client get somekey{t}
         $watching_client read
         $watching_client exec
+        # wait for exec to be called.
+        wait_for_condition 50 10 {
+            [llength [lsearch -all [split [string trim [r client list]] "\r\n"] *cmd=exec*]] == 1
+        } else {
+            fail "$cmd was not called"
+        }
+
         # Blocked BLPOPLPUSH may create problems, unblock it.
         r lpush srclist{t} element
         set res [$watching_client read]
@@ -1133,7 +1116,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     } {somevalue}
 
     test {BRPOPLPUSH timeout} {
-      set rd [redis_deferring_client]
+      set rd [valkey_deferring_client]
 
       $rd brpoplpush foo_list{t} bar_list{t} 1
       wait_for_blocked_clients_count 1
@@ -1148,7 +1131,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         r select 1
         r rpush k hello
         r select 9
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd brpop k 5
         wait_for_blocked_clients_count 1
         r swapdb 1 9
@@ -1162,7 +1145,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         r select 1
         r rpush k hello
         r pexpire k 100
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd deferred 0
         $rd select 9
         set id [$rd client id]
@@ -1179,7 +1162,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         assert_equal {} [$rd read]
         $rd deferred 0
         # We want to force key deletion to be propagated to the replica 
-        # in order to verify it was expiered on the replication stream. 
+        # in order to verify it was expired on the replication stream.
         $rd set somekey1 someval1
         $rd exists k
         r set somekey2 someval2
@@ -1193,7 +1176,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
             {swapdb 1 9}
             {select 9}
             {set somekey1 someval1}
-            {del k}
+            {unlink k}
             {select 1}
             {set somekey2 someval2}
         }
@@ -1208,7 +1191,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         r flushall
         r debug set-active-expire 0
 
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd client id
         set id [$rd read]
         $rd brpop k 0
@@ -1227,7 +1210,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         r client unblock $id
         assert_equal {} [$rd read]
         # We want to force key deletion to be propagated to the replica 
-        # in order to verify it was expiered on the replication stream. 
+        # in order to verify it was expired on the replication stream.
         $rd exists k
         assert_equal {0} [$rd read]
         assert_replication_stream $repl {
@@ -1237,7 +1220,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
             {rpush k hello}
             {pexpireat k *}
             {exec}
-            {del k}
+            {unlink k}
         }
         close_replication_stream $repl
         # Restore server and client state
@@ -1248,7 +1231,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     test {BLPOP unblock but the key is expired and then block again - reprocessing command} {
         r flushall
         r debug set-active-expire 0
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
 
         set start [clock milliseconds]
         $rd blpop mylist 1
@@ -1275,7 +1258,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
 
 foreach {pop} {BLPOP BLMPOP_LEFT} {
     test "$pop when new key is moved into place" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         r del foo{t}
 
         bpop_command $rd $pop foo{t} 0
@@ -1288,7 +1271,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
     } {foo{t} hij}
 
     test "$pop when result key is created by SORT..STORE" {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
 
         # zero out list from previous test without explicit delete
         r lpop foo{t}
@@ -1315,7 +1298,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         
     foreach {pop} {BLPOP BRPOP BLMPOP_LEFT BLMPOP_RIGHT} {
         test "$pop: with single empty list argument" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1
             bpop_command $rd $pop blist1 1
             wait_for_blocked_client
@@ -1326,14 +1309,14 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         }
 
         test "$pop: with negative timeout" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             bpop_command $rd $pop blist1 -1
             assert_error "ERR *is negative*" {$rd read}
             $rd close
         }
 
         test "$pop: with non-integer timeout" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1
             bpop_command $rd $pop blist1 0.1
             r rpush blist1 foo
@@ -1345,7 +1328,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         test "$pop: with zero timeout should block indefinitely" {
             # To test this, use a timeout of 0 and wait a second.
             # The blocking pop should still be waiting for a push.
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             bpop_command $rd $pop blist1 0
             wait_for_blocked_client
             r rpush blist1 foo
@@ -1356,7 +1339,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         test "$pop: with 0.001 timeout should not block indefinitely" {
             # Use a timeout of 0.001 and wait for the number of blocked clients to equal 0.
             # Validate the empty read from the deferring client.
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             bpop_command $rd $pop blist1 0.001
             wait_for_blocked_clients_count 0
             assert_equal {} [$rd read]
@@ -1364,7 +1347,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         }
 
         test "$pop: second argument is not a list" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1{t} blist2{t}
             r set blist2{t} nolist{t}
             bpop_command_two_key $rd $pop blist1{t} blist2{t} 1
@@ -1373,7 +1356,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         }
 
         test "$pop: timeout" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1{t} blist2{t}
             bpop_command_two_key $rd $pop blist1{t} blist2{t} 1
             wait_for_blocked_client
@@ -1382,7 +1365,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
         }
 
         test "$pop: arguments are empty" {
-            set rd [redis_deferring_client]
+            set rd [valkey_deferring_client]
             r del blist1{t} blist2{t}
 
             bpop_command_two_key $rd $pop blist1{t} blist2{t} 1
@@ -1417,7 +1400,7 @@ foreach {pop} {BLPOP BLMPOP_LEFT} {
 }
 
     test {BLMPOP propagate as pop with count command to replica} {
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         set repl [attach_to_replication_stream]
 
         # BLMPOP without being blocked.
@@ -2033,8 +2016,8 @@ foreach {type large} [array get largevalue] {
     }
 
     test "Regression for bug 593 - chaining BRPOPLPUSH with other blocking cmds" {
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
 
         $rd1 brpoplpush a{t} b{t} 0
         $rd1 brpoplpush a{t} b{t} 0
@@ -2049,7 +2032,7 @@ foreach {type large} [array get largevalue] {
 
     test "BLPOP/BLMOVE should increase dirty" {
         r del lst{t} lst1{t}
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
 
         set dirty [s rdb_changes_since_last_save]
         $rd blpop lst{t} 0
@@ -2073,7 +2056,7 @@ foreach {type large} [array get largevalue] {
 foreach {pop} {BLPOP BLMPOP_RIGHT} {
     test "client unblock tests" {
         r del l
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd client id
         set id [$rd read]
 
@@ -2285,8 +2268,8 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
     } {12 0 9223372036854775808 2147483647 32767 127}
     
     test "Unblock fairness is kept while pipelining" {
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
         
         # delete the list in case already exists
         r del mylist
@@ -2320,9 +2303,9 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
     }
     
     test "Unblock fairness is kept during nested unblock" {
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
-        set rd3 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
+        set rd3 [valkey_deferring_client]
         
         # delete the list in case already exists
         r del l1{t} l2{t} l3{t}
@@ -2358,7 +2341,7 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
         r del mylist
         
         # create a test client
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         
         # reset the server stats
         r config resetstat
@@ -2381,7 +2364,7 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
         r del mylist
         
         # create a test client
-        set rd [redis_deferring_client]
+        set rd [valkey_deferring_client]
         $rd client id
         set id [$rd read]
 
@@ -2404,9 +2387,9 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
         r del src{t} dst{t} key1{t} key2{t} key3{t}
         set repl [attach_to_replication_stream]
 
-        set rd1 [redis_deferring_client]
-        set rd2 [redis_deferring_client]
-        set rd3 [redis_deferring_client]
+        set rd1 [valkey_deferring_client]
+        set rd2 [valkey_deferring_client]
+        set rd3 [valkey_deferring_client]
 
         $rd1 blmove src{t} dst{t} left right 0
         wait_for_blocked_clients_count 1
@@ -2447,27 +2430,4 @@ foreach {pop} {BLPOP BLMPOP_RIGHT} {
         close_replication_stream $repl
     } {} {needs:repl}
 
-    test "Blocking timeout following PAUSE should honor the timeout" {
-        # cleanup first
-        r del mylist
-
-        # create a test client
-        set rd [redis_deferring_client]
-
-        # first PAUSE all writes for a very long time
-        r client pause 10000000000000 write
-
-        # block a client on the list
-        $rd BLPOP mylist 1
-        wait_for_blocked_clients_count 1
-
-        # now unpause the writes
-        r client unpause
-
-        # client should time-out
-        wait_for_blocked_clients_count 0
-
-        $rd close
-    }
-
 } ;# stop servers
